@@ -61,8 +61,8 @@ class PmAgentClient(metaclass=ABCMeta):
 
     async def _idle(self):  # todo: не мешать другим процессам, обновлять на другой вкладке?
         while (await User.get(username_id=self.uid)).status >= UserStatus.ACTIVE:
-            await self.page.wait_for_timeout(15 * 1000)
-            if self.last_active < datetime.now() - timedelta(seconds=20):
+            await self.page.wait_for_timeout(30 * 1000)
+            if self.last_active < datetime.now() - timedelta(minutes=1):
                 await self.page.reload()
                 self.last_active = datetime.now()
         await self.bot.send(self.norm + " stoped", self.uid)
