@@ -1,15 +1,18 @@
 import json
 import logging
 import re
-from datetime import datetime, timezone
+from datetime import datetime, timezone, timedelta
 from uuid import uuid4
 
 import websockets
-from asyncio import run
+from asyncio import run, sleep
 from decimal import Decimal
 
 from playwright.async_api import async_playwright
+from pydantic import ValidationError
 from pyro_client.client.file import FileClient
+from tortoise.exceptions import IntegrityError
+from tortoise.timezone import now
 from tortoise.transactions import in_transaction
 
 from xync_client.Abc.PmAgent import PmAgentClient
@@ -78,7 +81,11 @@ class InAgentClient(BaseInAgentClient):
                         case "OTC_ORDER_STATUS":
                             match data["type"]:
                                 case "STATUS_CHANGE":
-                                    upd = StatusChange.model_validate(data["data"])
+                                    try:
+                                        upd = StatusChange.model_validate(data["data"])
+                                    except ValidationError as e:
+                                        logging.error(e)
+                                        logging.error(data["data"])
                                     order = self.agent_client.api.get_order_details(orderId=upd.id)
                                     order = OrderFull.model_validate(order["result"])
                                     order_db = await models.Order.get_or_none(
@@ -91,7 +98,7 @@ class InAgentClient(BaseInAgentClient):
                                             await self.money_upd(order_db)
                                             if upd.side:  # я покупатель - ждем мою оплату
                                                 dest = order.paymentTermList[0].accountNo
-                                                if not re.match(r"^([Pp])\d{8,10}$", dest):
+                                                if not re.match(r"^([PpРр])\d{7,10}\b", dest):
                                                     continue
                                                 await order_db.fetch_related("ad__pair_side__pair", "cred__pmcur__cur")
                                                 await self.send_payment(order_db, dest)
@@ -110,16 +117,52 @@ class InAgentClient(BaseInAgentClient):
                                                         f"Order {order.id} created at {order.createDate}, not paid yet"
                                                     )
                                                     continue
-                                                t, is_new = await models.Transfer.update_or_create(
-                                                    dict(
-                                                        amount=int(float(order.amount) * 100),
-                                                        accepted_at=datetime.now(timezone.utc),
-                                                    ),
-                                                    order=order_db,
-                                                    # pm_id=order_db.cred.pmcur.pm_id,
-                                                    pmid=tid,
+                                                try:
+                                                    t, is_new = await models.Transfer.update_or_create(
+                                                        dict(
+                                                            amount=int(float(order.amount) * 100),
+                                                            order=order_db,
+                                                        ),
+                                                        pmid=tid,
+                                                    )
+                                                except IntegrityError as e:
+                                                    logging.error(tid)
+                                                    logging.error(order)
+                                                    logging.exception(e)
+
+                                                if not is_new:  # если по этому платежу уже отпущен другая продажа
+                                                    continue
+
+                                                # если висят незавершенные продажи с такой же суммой
+                                                pos = (await self.agent_client.get_orders_active(1))["result"]
+                                                pos = [
+                                                    o
+                                                    for o in pos.get("items", [])
+                                                    if (
+                                                        o["amount"] == order.amount
+                                                        and o["id"] != upd.id
+                                                        and o.paymentTermList[0].accountNo
+                                                        == order.paymentTermList[0].accountNo
+                                                        and int(order.createDate) < int(o["createDate"]) + 3600 * 000
+                                                    )
+                                                ]
+                                                curex = await models.CurEx.get(
+                                                    cur__ticker=order.currencyId, ex=self.agent_client.ex_client.ex
                                                 )
-                                                if not is_new:
+                                                pos_db = await models.Order.filter(
+                                                    exid__not=order.id,
+                                                    cred_id=order_db.cred_id,
+                                                    amount=int(float(order.amount) * 10**curex.scale),
+                                                    status__not_in=[OrderStatus.completed, OrderStatus.canceled],
+                                                    created_at__gt=now() - timedelta(hours=1),
+                                                )
+                                                if pos or pos_db:
+                                                    await self.agent_client.ex_client.bot.send(
+                                                        f"[Duplicate amount!]"
+                                                        f"(https://www.bybit.com/ru-RU/p2p/orderList/{order.id})",
+                                                        self.agent_client.actor.person.user.username_id,
+                                                    )
+                                                    logging.warning("Duplicate amount!")
                                                     continue
 
                                                 # !!! ОТПРАВЛЯЕМ ДЕНЬГИ !!!
@@ -191,19 +234,19 @@ class InAgentClient(BaseInAgentClient):
                                     ).prefetch_related("ad__pair_side__pair", "cred__pmcur__cur"):
                                         im_taker = order_db.taker_id == self.agent_client.actor.id
                                         im_buyer = order_db.ad.pair_side.is_sell == im_taker
-                                        if order_db.ad.auto_msg != upd.message and upd.roleType not in ("sys", "alarm"):
+                                        if order_db.ad.auto_msg != upd.message and upd.roleType == "user":
                                             msg, _ = await models.Msg.update_or_create(
                                                 {
                                                     "to_maker": upd.userId == self.agent_client.actor.exid and im_taker,
+                                                    "sent_at": datetime.fromtimestamp(float(upd.createDate) / 1000),
                                                 },
-                                                sent_at=datetime.fromtimestamp(float(upd.createDate) / 1000),
                                                 txt=upd.message,
                                                 order=order_db,
                                             )
                                             if not upd.message:
                                                 ...
-                                            if im_buyer and re.match(r"^P\d{8,10}$", upd.message):
-                                                await self.send_payment(order_db, upd.message)
+                                            if im_buyer and (g := re.match(r"^[PpРр]\d{7,10}\b", upd.message)):
+                                                await self.send_payment(order_db, g.group())
                                 case "READ":
                                     upd = Read.model_validate(data["data"])
                                     # if upd.status not in (StatusWs.created, StatusWs.canceled, 10, StatusWs.completed):
@@ -242,7 +285,7 @@ class InAgentClient(BaseInAgentClient):
 
     async def money_upd(self, order_db: models.Order):
         # обновляем остаток монеты
-        await order_db.fetch_related("ad__pair_side__pair", "cred")
+        await order_db.fetch_related("ad__pair_side__pair", "cred", "transfer")
         ass = await models.Asset.get(
             addr__coin_id=order_db.ad.pair_side.pair.coin_id, addr__actor=self.agent_client.actor
         )
@@ -292,7 +335,10 @@ class InAgentClient(BaseInAgentClient):
             )
             # проверяем не отправляли ли мы уже перевод по этому ордеру
             if t := await models.Transfer.get_or_none(order=order_db, amount=order_db.amount):
-                await pma.bot.send(f"Order# {order_db.exid}: Double send {fmt_am}{cur} to {dest} #{t.pmid}!")
+                await pma.bot.send(
+                    f"Order# {order_db.exid}: Double send {fmt_am}{cur} to {dest} #{t.pmid}!",
+                    self.agent_client.actor.person.user.username_id,
+                )
                 raise Exception(f"Order# {order_db.exid}: Double send {fmt_am}{cur} to {dest} #{t.pmid}!")
 
             # ставим в бд статус "оплачен"
@@ -302,18 +348,24 @@ class InAgentClient(BaseInAgentClient):
             # отправляем деньги
             tid, img, rest_amount = await pma.send(dest=dest, amount=fmt_am, cur=cur)
             # создаем перевод в бд
-            t, _ = await models.Transfer.update_or_create(
-                {"amount": order_db.amount},
-                order=order_db,
-                pmid=tid,
-            )
-            # отправляем продавцу чек
-            if res := self.agent_client.api.upload_chat_file(upload_file=f"tmp/{dest}.png").get("result"):
-                self.agent_client.api.send_chat_message(
-                    orderId=str(order_db.exid), contentType="pic", message=res["url"], msgUuid=uuid4().hex
-                )
+            t, _ = await models.Transfer.update_or_create({"order": order_db, "amount": order_db.amount}, pmid=tid)
+            await self.send_receipt(str(order_db.exid), tid)  # отправляем продавцу чек
             logging.info(f"Order {order_db.exid} PAID at {datetime.now()}: {fmt_am}!")
 
+    async def send_receipt(self, oexid: str, tid: int) -> tuple[PmAgentClient | None, models.CredEx] | None:
+        try:
+            if res := self.agent_client.api.upload_chat_file(upload_file=f"tmp/{tid}.png").get("result"):
+                await sleep(0.5)
+                self.agent_client.api.send_chat_message(
+                    orderId=oexid, contentType="pic", message=res["url"], msgUuid=uuid4().hex
+                )
+        except Exception as e:
+            logging.error(e)
+        await sleep(0.5)
+        self.agent_client.api.send_chat_message(
+            orderId=oexid, contentType="str", message=f"#{tid}", msgUuid=uuid4().hex
+        )
+
     async def get_pma_by_cdex(self, order: OrderFull) -> tuple[PmAgentClient | None, models.CredEx] | None:
         cdxs = await models.CredEx.filter(
             ex=self.agent_client.ex_client.ex,
@@ -360,13 +412,17 @@ async def main():
     )
 
     async with FileClient(TOKEN) as b:
+        b: FileClient
         cl: InAgentClient = actor.in_client(b)
         # await cl.agent_client.export_my_ads()
         payeer_cl = Client(actor.person.user.username_id)
         for pma in actor.person.user.pm_agents:
             cl.pmacs[pma.pm_id] = await payeer_cl.start(await async_playwright().start(), False)
-
-        _ = await cl.start_listen()
+        try:
+            _ = await cl.start_listen()
+        except Exception as e:
+            await b.send("😱Bybit InAgent CRASHED!!!😱", actor.person.user.username_id)
+            await b.send(e.__repr__(), actor.person.user.username_id)
         await cl.agent_client.close()
 
 
