@@ -7,12 +7,14 @@ from datetime import datetime, timedelta, timezone
 from difflib import SequenceMatcher
 from enum import IntEnum
 from http.client import HTTPException
+from math import floor
 from typing import Literal
 
 import pyotp
 from asyncpg import ConnectionDoesNotExistError
 from bybit_p2p import P2P
 from bybit_p2p._exceptions import FailedRequestError
+from payeer_api import PayeerAPI
 from pyro_client.client.file import FileClient
 from tortoise import BaseDBAsyncClient
 from tortoise.exceptions import IntegrityError
@@ -88,6 +90,8 @@ class AgentClient(BaseAgentClient):  # Bybit client
     def __init__(self, actor: Actor, bot: FileClient, **kwargs):
         super().__init__(actor, bot, **kwargs)
         self.api = P2P(testnet=False, api_key=actor.agent.auth["key"], api_secret=actor.agent.auth["sec"])
+        self.hist: dict = None
+        self.completed_orders: list[int] = None
 
     """ Private METHs"""
 
@@ -393,8 +397,16 @@ class AgentClient(BaseAgentClient):  # Bybit client
             },
         )
 
-    def get_orders_active(self, begin_time: int, end_time: int, status: int, side: int, token_id: str):
+    def my_reviews(self):
         return self._post(
+            "/x-api/fiat/otc/order/appraiseList",
+            {"makerUserId": self.actor.exid, "page": "1", "size": "10", "appraiseType": "1"},  # "0" - bad
+        )
+
+    async def get_orders_active(
+        self, side: int = None, status: int = None, begin_time: int = None, end_time: int = None, token_id: str = None
+    ):
+        return await self._post(
             "/fiat/otc/order/pending/simplifyList",
             {
                 "status": status,
@@ -440,9 +452,28 @@ class AgentClient(BaseAgentClient):  # Bybit client
         if not ad_db:
             ...
         ecredex: CredEpyd = order.confirmedPayTerm
+
         if ecredex.paymentType == 0 and im_maker and order.side:
             ecredex = order.paymentTermList[0]
         if ecredex.paymentType:
+            if ecredex.paymentType == 51:
+                ecredex.accountNo = ecredex.accountNo.replace("p", "P").replace("р", "P").replace("Р", "P")
+                if not re.match(r"^([Pp])\d{7,10}$", ecredex.accountNo):
+                    msgs = self.api.get_chat_messages(orderId=order.id, size=100)["result"]["result"]
+                    msgs = [m["message"] for m in msgs if m["roleType"] == "user" and m["userId"] == order.targetUserId]
+                    msgs = [g.group() for m in msgs if (g := re.match(r"([PpРр])\d{7,10}\b", m))]
+                    crd = await models.Cred.get_or_none(
+                        detail=ecredex.accountNo, credexs__exid=ecredex.id, credexs__ex=self.ex_client.ex
+                    )
+                    if not msgs and re.match(r"^\d{7,10}$", ecredex.accountNo):
+                        ecredex.accountNo = "P" + ecredex.accountNo
+                    elif msgs:
+                        ecredex.accountNo = msgs[-1]
+                    else:
+                        ...
+                    if crd:
+                        crd.detail = ecredex.accountNo
+                        await crd.save(update_fields=["detail"])
             if not (credex := await models.CredEx.get_or_none(exid=ecredex.id, ex=self.ex_client.ex)):
                 # cur_id = await Cur.get(ticker=ad.currencyId).values_list('id', flat=True)
                 # await self.cred_epyd2db(ecredex, ad_db.maker.person_id, cur_id)
@@ -496,16 +527,55 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 "amount": float(order.amount) * 10**cur_scale,
                 "quantity": float(order.quantity) * 10**coin_scale,
                 "status": OrderStatus[Status(order.status).name],
-                "created_at": int(order.createDate[:-3]),
-                "payed_at": order.transferDate != "0" and int(order.transferDate[:-3]) or None,
-                "confirmed_at": Status(order.status) == Status.completed and int(order.updateDate[:-3]) or None,
-                "appealed_at": order.status == 30 and int(order.updateDate[:-3]) or None,
+                "created_at": ms2utc(order.createDate),
+                "payed_at": order.transferDate != "0" and ms2utc(order.transferDate) or None,
+                "confirmed_at": Status(order.status) == Status.completed and ms2utc(order.transferDate) or None,
+                "appealed_at": order.status == 30 and ms2utc(order.transferDate) or None,
                 "cred_id": ecredex.paymentType and credex.cred_id or None,
                 "taker": taker,
                 "ad": ad_db,
             },
             exid=order.id,
         )
+        if order.status == Status.completed and ecredex.paymentType == 51:
+            await odb.fetch_related("cred", "transfer")
+            if odb.cred.detail != ecredex.accountNo:
+                ...
+            frm = (odb.created_at + timedelta(minutes=180 - 1)).isoformat(sep=" ").split("+")[0]
+            to = ((odb.payed_at or odb.created_at) + timedelta(minutes=180 + 30)).isoformat(sep=" ").split("+")[0]
+            tsa = [
+                t
+                for tid, t in self.hist.items()
+                if (ecredex.accountNo == t["to"] and t["from"] != "@merchant" and frm < t["date"] < to)
+            ]
+            buyer_person = (
+                self.actor.person
+                if not order.side
+                else await self.person_upsert(order.buyerRealName, int(order.targetUserId))
+            )
+            ts = [t for t in tsa if floor(fa := float(order.amount)) <= float(t["creditedAmount"]) <= round(fa)]
+            if len(ts) != 1:
+                if len(tsa) > 1:
+                    summ = sum(float(t["creditedAmount"]) for t in tsa)
+                    if floor(fa) <= summ <= round(fa):
+                        for tr in tsa:
+                            am = int(float(tr["creditedAmount"]) * 100)
+                            await models.Transfer.create(
+                                pmid=tr["id"], order=odb, amount=am, sender_acc=tr["from"], created_at=tr["date"]
+                            )
+            else:
+                bcred, _ = await models.Cred.get_or_create(
+                    {"detail": ts[0]["from"]}, person=buyer_person, pmcur_id=odb.cred.pmcur_id
+                )
+                am = int(float(ts[0]["creditedAmount"]) * 100)
+                try:
+                    await models.Transfer.create(
+                        pmid=ts[0]["id"], order=odb, amount=am, sender_acc=ts[0]["from"], created_at=ts[0]["date"]
+                    )
+                except IntegrityError as e:
+                    logging.error(e)
+            ...
+
         await odb.fetch_related("ad")
         return odb
 
@@ -523,8 +593,8 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 page=page,
                 # status=status,  # 50 - завершено
                 # tokenId=token_id,
-                beginTime=begin_time,
-                endTime=end_time,
+                # beginTime=begin_time,
+                # endTime=end_time,
                 # side=side, # 1 - продажа, 0 - покупка
                 size=30,
             )
@@ -534,6 +604,8 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 await self.get_api_orders(page, begin_time, end_time)  # , status, side, token_id)
         ords = {int(o["id"]): OrderItem.model_validate(o) for o in lst["result"]["items"]}
         for oid, o in ords.items():
+            if o.status != Status.completed.value or oid in self.completed_orders:
+                continue
             fo = self.api.get_order_details(orderId=o.id)
             order = OrderFull.model_validate(fo["result"])
             order_db = await self.create_order(order)
@@ -557,6 +629,10 @@ class AgentClient(BaseAgentClient):  # Bybit client
         if len(ords) == 30:
             await self.get_api_orders(page + 1, begin_time, end_time, status, side, token_id)
 
+    # async def order_stat(self, papi: PayeerAPI):
+    #     for t in papi.history():
+    #         os = self.api.get_orders(page=1, size=30)
+
     async def mad_upd(self, mad: Ad, attrs: dict, cxids: list[str]):
         if not [setattr(mad, k, v) for k, v in attrs.items() if getattr(mad, k) != v]:
             print(end="v" if mad.side else "^", flush=True)
@@ -661,10 +737,15 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 volume = fiat.amount / race.road.ad.price
             else:  # гонка в стакане покупки - мы продаем МОНЕТУ за фиат
                 asset = await models.Asset.get(addr__actor=self.actor, addr__coin_id=coinex.coin_id)
-                volume = (asset.free - (asset.freeze or 0) - (asset.lock or 0)) * 10**-coinex.scale
+                volume = asset.free * 10**-coinex.scale
             volume = str(round(volume, coinex.scale))
 
-            ads: list[Ad] = await self.ads(coinex, curex, taker_side, pmexs)
+            try:
+                ads: list[Ad] = await self.ads(coinex, curex, taker_side, pmexs)
+            except Exception:
+                await sleep(1)
+                ads: list[Ad] = await self.ads(coinex, curex, taker_side, pmexs)
+
             if race.vm_filter:
                 ads = [ad for ad in ads if "VA" in ad.authTag]
             self.overprice_filter(ads, race.ceil * 10**-curex.scale, k)  # обрезаем сверху все ads дороже нашего потолка
@@ -755,12 +836,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
                         raise e
                 elif ExcCode(e.status_code) == ExcCode.InsufficientBalance:
                     asset = await models.Asset.get(addr__actor=self.actor, addr__coin_id=coinex.coin_id)
-                    req.quantity = str(
-                        round(
-                            (asset.free - (asset.freeze or 0) - (asset.lock or 0)) * 10**-coinex.scale,
-                            coinex.coin.scale or coinex.scale,
-                        )
-                    )
+                    req.quantity = str(round(asset.free * 10**-coinex.scale, coinex.scale))
                     _res = self.ad_upd(req)
                 elif ExcCode(e.status_code) == ExcCode.RareLimit:
                     sad = [
@@ -852,7 +928,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 volume = min(volume, max(fiats, key=lambda f: f.target - f.amount).amount / ceil)
             else:  # гонка в стакане покупки - мы продаем МОНЕТУ за фиат
                 asset = await models.Asset.get(addr__actor=self.actor, addr__coin_id=coinex.coin_id)
-                volume = min(volume, asset.free - (asset.freeze or 0) - (asset.lock or 0))
+                volume = min(volume, asset.free)
         volume = str(round(volume, coinex.coin.scale))
         ps = await PairSide.get(
             is_sell=taker_side,
@@ -1109,6 +1185,10 @@ class AgentClient(BaseAgentClient):  # Bybit client
         self.tree = tree
 
 
+def ms2utc(msk_ts_str: str):
+    return datetime.fromtimestamp(int(msk_ts_str) / 1000, timezone(timedelta(hours=3), name="MSK"))
+
+
 def get_sim(s1, s2) -> int:
     sim = int((SequenceMatcher(None, s1, s2).ratio() - 0.6) * 10_000)
     return sim if sim > 0 else 0
@@ -1184,94 +1264,91 @@ async def main():
     actor = (
         await models.Actor.filter(ex_id=4, agent__isnull=False).prefetch_related("ex", "agent", "person__user").first()
     )
-    async with FileClient(TOKEN) as b:
-        b: FileClient
-        # b.add_handler(MessageHandler(cond_start_handler, command("cond")))
-        cl: AgentClient = actor.client(b)
+    filebot = FileClient(TOKEN)
+    await filebot.start()
+    # b.add_handler(MessageHandler(cond_start_handler, command("cond")))
+    cl: AgentClient = actor.client(filebot)
 
-        # await cl.ex_client.set_pairs()
-        # await cl.ex_client.set_pms()
-        # await cl.set_creds()
-        # await cl.export_my_ads()
+    # await cl.ex_client.set_pairs()
+    # await cl.ex_client.set_pms()
+    # await cl.set_creds()
+    # await cl.export_my_ads()
 
-        # создание гонок по мои активным объявам:
-        # for ma in cl.my_ads():
-        #     my_ad = await models.MyAd.get(ad__exid=ma.id).prefetch_related('ad__pms', 'ad__pair_side__pair')
-        #     race, _ = await models.Race.update_or_create(
-        #         {"started": True, "vm_filter": True, "target_place": 5},
-        #         road=my_ad
-        #     )
+    # создание гонок по мои активным объявам:
+    # for ma in cl.my_ads():
+    #     my_ad = await models.MyAd.get(ad__exid=ma.id).prefetch_related('ad__pms', 'ad__pair_side__pair')
+    #     race, _ = await models.Race.update_or_create(
+    #         {"started": True, "vm_filter": True, "target_place": 5},
+    #         road=my_ad
+    #     )
 
-        # for name in names:
-        #     s, _ = await models.Synonym.update_or_create(typ=SynonymType.name, txt=name)
-        #     await s.curs.add(rub.cur)
+    # for name in names:
+    #     s, _ = await models.Synonym.update_or_create(typ=SynonymType.name, txt=name)
+    #     await s.curs.add(rub.cur)
 
-        # пока порешали рейс-кондишн, очищаем сиротские условия при каждом запуске
-        # [await c.delete() for c in await Cond.filter(ads__isnull=True)]
-        cl.all_conds = {
-            c.id: (c.raw_txt, {a.maker.exid for a in c.ads}) for c in await Cond.all().prefetch_related("ads__maker")
-        }
-        for curr, old in await CondSim.filter().values_list("cond_id", "cond_rel_id"):
-            cl.cond_sims[curr] = old
-            cl.rcond_sims[old] |= {curr}
+    # пока порешали рейс-кондишн, очищаем сиротские условия при каждом запуске
+    # [await c.delete() for c in await Cond.filter(ads__isnull=True)]
+    cl.all_conds = {
+        c.id: (c.raw_txt, {a.maker.exid for a in c.ads}) for c in await Cond.all().prefetch_related("ads__maker")
+    }
+    for curr, old in await CondSim.filter().values_list("cond_id", "cond_rel_id"):
+        cl.cond_sims[curr] = old
+        cl.rcond_sims[old] |= {curr}
 
-        cl.build_tree()
-        a = set()
+    cl.build_tree()
+    a = set()
 
-        def check_tree(tre):
-            for p, c in tre.items():
-                a.add(p)
-                check_tree(c)
+    def check_tree(tre):
+        for p, c in tre.items():
+            a.add(p)
+            check_tree(c)
 
-        for pr, ch in cl.tree.items():
-            check_tree(ch)
-        if ct := set(cl.tree.keys()) & a:
-            logging.exception(f"cycle cids: {ct}")
+    for pr, ch in cl.tree.items():
+        check_tree(ch)
+    if ct := set(cl.tree.keys()) & a:
+        logging.exception(f"cycle cids: {ct}")
 
-        # await cl.get_api_orders(43, 1741294800000, 1749157199999)
+    pauth = (await models.PmAgent[1]).auth
+    papi = PayeerAPI(pauth["email"], pauth["api_id"], pauth["api_sec"])
+    hist: dict = papi.history(count=1000)
+    hist |= papi.history(count=1000, append=list(hist.keys())[-1])
+    hist |= papi.history(count=1000, append=list(hist.keys())[-1])
+    cl.hist = hist
 
-        races = await models.Race.filter(started=True).prefetch_related(
-            "road__ad__pair_side__pair__cur",
-            "road__ad__pms",
-        )
-        tasks = [asyncio.create_task(cl.racing(race), name=f"Rc{race.id}") for race in races]
-        # await cl.actual_cond()
+    cl.completed_orders = await models.Order.filter(status=OrderStatus.completed, transfer__isnull=False).values_list(
+        "exid", flat=True
+    )
+    await cl.get_api_orders()  # 43, 1741294800000, 1749157199999)
+
+    races = await models.Race.filter(started=True).prefetch_related(
+        "road__ad__pair_side__pair__cur",
+        "road__ad__pms",
+    )
+    tasks = [asyncio.create_task(cl.racing(race), name=f"Rc{race.id}") for race in races]
+    # await cl.actual_cond()
+    try:
         await gather(
             *tasks
             # cl.get_api_orders(),  # 10, 1738357200000, 1742504399999
-            # cl.racing(usdt, rub, False, ["volet"], 83.88),  # гонка в стакане покупки - мы продаем
-            # cl.racing(usdt, rub, True, ["volet"], 81.51),  # гонка в стакане продажи - мы покупаем
-            # cl.racing(usdt, rub, False, ["payeer"], 81.49),  # гонка в стакане покупки - мы продаем
-            # cl.racing(usdt, rub, True, ["payeer"], 80.7),  # гонка в стакане продажи - мы покупаем
-            # cl.racing(eth, rub, False, ["volet"], 300_000),
-            # cl.racing(eth, rub, True, ["volet"], 296_000),
-            # cl.racing(eth, rub, False, ["payeer"], 300_000),
-            # cl.racing(eth, rub, True, ["payeer"], 296_000),
-            # cl.racing(btc, rub, False, ["volet"], 9_800_000),
-            # cl.racing(btc, rub, True, ["volet"], 9_700_000),
-            # cl.racing(btc, rub, False, ["payeer"], 9_800_000),
-            # cl.racing(btc, rub, True, ["payeer"], 9_700_000),
-            # cl.racing(usdc, rub, False, ["volet"], 80.98),
-            # cl.racing(usdc, rub, True, ["volet"], 80.01),
-            # cl.racing(usdc, rub, False, ["payeer"], 80.98),
-            # cl.racing(usdc, rub, True, ["payeer"], 80.01),
-            # cl.parse_ads(usdt, rub, False, ceil=81, volume=360),
-            # cl.parse_ads(usdt, rub, True, ceil=80, volume=360),
         )
-
-        # bor = BaseOrderReq(
-        #     ad_id="1861440060199632896",
-        #     # asset_amount=40,
-        #     fiat_amount=3000,
-        #     amount_is_fiat=True,
-        #     is_sell=False,
-        #     cur_exid=rub.exid,
-        #     coin_exid=usdt.exid,
-        #     coin_scale=usdt.coin.scale,
-        # )
-        # res: OrderResp = await cl.order_request(bor)
-        # await cl.cancel_order(res.orderId)
-        await cl.close()
+    except Exception as e:
+        await filebot.send("🤬Bybit agent CRASHED!!!🤬", actor.person.user.username_id)
+        await filebot.send(e.__repr__(), actor.person.user.username_id)
+        raise e
+    # bor = BaseOrderReq(
+    #     ad_id="1861440060199632896",
+    #     # asset_amount=40,
+    #     fiat_amount=3000,
+    #     amount_is_fiat=True,
+    #     is_sell=False,
+    #     cur_exid=rub.exid,
+    #     coin_exid=usdt.exid,
+    #     coin_scale=usdt.coin.scale,
+    # )
+    # res: OrderResp = await cl.order_request(bor)
+    # await cl.cancel_order(res.orderId)
+    await filebot.stop()
+    await cl.close()
 
 
 if __name__ == "__main__":
